=begin
  Copyright (C) 2006 Sam Roberts

  This library is free software; you can redistribute it and/or modify it
  under the same terms as the ruby language itself, see the file COPYING for
  details.
=end

# the existence of this file is a hack to support users or rubygems

require 'vpim/icalendar'
require 'vpim/vcard'

