#!/usr/bin/env ruby

require 'test/unit'

$:.unshift File.dirname($0)

require 'test_date.rb'
require 'test_dur.rb'
require 'test_field.rb'
require 'test_ical.rb'
#require 'test_rrule.rb' Doesn't work in PST/PDT...
require 'test_vcard.rb'

